#!/usr/bin/env bash

source "$NCTL"/sh/utils/main.sh
source "$NCTL"/sh/node/svc_"$NCTL_DAEMON_TYPE".sh
source "$NCTL"/sh/assets/upgrade.sh

unset LOG_LEVEL
unset NODE_ID
unset PROTOCOL_VERSION
unset ACTIVATE_ERA

for ARGUMENT in "$@"; do
    KEY=$(echo "$ARGUMENT" | cut -f1 -d=)
    VALUE=$(echo "$ARGUMENT" | cut -f2 -d=)
    case "$KEY" in
        version) PROTOCOL_VERSION=${VALUE} ;;
        era) ACTIVATE_ERA=${VALUE} ;;
        loglevel) LOG_LEVEL=${VALUE} ;;
        *) echo "Unknown argument '${KEY}'. Use 'version', 'era' or 'loglevel'." && exit 1 ;;
    esac
done

LOG_LEVEL=${LOG_LEVEL:-$RUST_LOG}
LOG_LEVEL=${LOG_LEVEL:-debug}
export RUST_LOG=$LOG_LEVEL

function do_upgrade() {
    local PROTOCOL_VERSION=${1}
    local ACTIVATE_ERA=${2}
    local NODE_COUNT=${3:-5}

    for NODE_ID in $(seq 1 $((NODE_COUNT * 2))); do
        _upgrade_node "$PROTOCOL_VERSION" "$ACTIVATE_ERA" "$NODE_ID"
    done
}

#######################################
# Upgrades all nodes in the network to a specified protocol version.
# Does not modify the chainspec file in any way that has an influence on the network,
# except for setting required entries for the upgrade to take place.
#######################################

#
# ----------------------------------------------------------------
# MAIN
# ----------------------------------------------------------------

# Upgrade node(s).
log "upgrade node(s) begins ... please wait"
do_upgrade "$PROTOCOL_VERSION" "$ACTIVATE_ERA"
log "upgrade node(s) complete"
